@@ -1,10 +1,8 @@
 ;;; calc-arith.el --- arithmetic functions for Calc
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2017 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
-;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
 
 ;; This file is part of GNU Emacs.
 
@@ -2250,7 +2248,7 @@
 
 (defun math-min-list (a b)
   (if b
-      (if (or (Math-anglep (car b)) (eq (car b) 'date)
+      (if (or (Math-anglep (car b)) (eq (caar b) 'date)
 	      (and (eq (car (car b)) 'intv) (math-intv-constp (car b)))
 	      (math-infinitep (car b)))
 	  (math-min-list (math-min a (car b)) (cdr b))
@@ -2280,7 +2278,7 @@
 
 (defun math-max-list (a b)
   (if b
-      (if (or (Math-anglep (car b)) (eq (car b) 'date)
+      (if (or (Math-anglep (car b)) (eq (caar b) 'date)
 	      (and (eq (car (car b)) 'intv) (math-intv-constp (car b)))
 	      (math-infinitep (car b)))
 	  (math-max-list (math-max a (car b)) (cdr b))
@@ -3067,5 +3065,4 @@
 
 (provide 'calc-arith)
 
-;; arch-tag: 6c396b5b-14c6-40ed-bb2a-7cc2e8111465
 ;;; calc-arith.el ends here
