@@ -3,54 +3,51 @@
 ;;; Code:
 
 
-;;;### (autoloads (diary-bahai-insert-yearly-entry diary-bahai-insert-monthly-entry
-;;;;;;  diary-bahai-insert-entry calendar-bahai-goto-date calendar-bahai-print-date
-;;;;;;  calendar-bahai-date-string) "cal-bahai" "cal-bahai.el" (19383
-;;;;;;  49278))
+;;;### (autoloads nil "cal-bahai" "cal-bahai.el" (22676 23208 557522
+;;;;;;  938000))
 ;;; Generated autoloads from cal-bahai.el
 
 (autoload 'calendar-bahai-date-string "cal-bahai" "\
-String of Bahá'í date of Gregorian DATE.
+String of Bahá’í date of Gregorian DATE.
 Defaults to today's date if DATE is not given.
 
 \(fn &optional DATE)" nil nil)
 
 (autoload 'calendar-bahai-print-date "cal-bahai" "\
-Show the Bahá'í calendar equivalent of the selected date.
+Show the Bahá’í calendar equivalent of the selected date.
 
 \(fn)" t nil)
 
 (autoload 'calendar-bahai-goto-date "cal-bahai" "\
-Move cursor to Bahá'í date DATE; echo Bahá'í date unless NOECHO is non-nil.
+Move cursor to Bahá’í date DATE; echo Bahá’í date unless NOECHO is non-nil.
 
 \(fn DATE &optional NOECHO)" t nil)
 
 (autoload 'diary-bahai-insert-entry "cal-bahai" "\
 Insert a diary entry.
-For the Bahá'í date corresponding to the date indicated by point.
+For the Bahá’í date corresponding to the date indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
 (autoload 'diary-bahai-insert-monthly-entry "cal-bahai" "\
 Insert a monthly diary entry.
-For the day of the Bahá'í month corresponding to the date indicated by point.
+For the day of the Bahá’í month corresponding to the date indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
 (autoload 'diary-bahai-insert-yearly-entry "cal-bahai" "\
 Insert an annual diary entry.
-For the day of the Bahá'í year corresponding to the date indicated by point.
+For the day of the Bahá’í year corresponding to the date indicated by point.
 Prefix argument ARG will make the entry nonmarking.
 
 \(fn ARG)" t nil)
 
 ;;;***
 
-;;;### (autoloads (calendar-chinese-goto-date calendar-chinese-print-date
-;;;;;;  calendar-chinese-date-string) "cal-china" "cal-china.el"
-;;;;;;  (19383 49278))
+;;;### (autoloads nil "cal-china" "cal-china.el" (22676 23208 557522
+;;;;;;  938000))
 ;;; Generated autoloads from cal-china.el
 
 (autoload 'calendar-chinese-date-string "cal-china" "\
@@ -70,12 +67,57 @@ Echo Chinese date unless NOECHO is non-nil.
 
 \(fn DATE &optional NOECHO)" t nil)
 
+(autoload 'diary-chinese-mark-entries "cal-china" "\
+Mark days in the calendar window that have Chinese date diary entries.
+Marks each entry in `diary-file' (or included files) visible in the calendar
+window.  See `diary-chinese-list-entries' for more information.
+
+This function is provided for use with `diary-nongregorian-marking-hook'.
+
+\(fn)" nil nil)
+
+(autoload 'diary-chinese-list-entries "cal-china" "\
+Add any Chinese date entries from the diary file to `diary-entries-list'.
+Chinese date diary entries must be prefixed by `diary-chinese-entry-symbol'
+\(normally a `C').  The same `diary-date-forms' govern the style
+of the Chinese calendar entries.  If a Chinese date diary entry begins with
+`diary-nonmarking-symbol', the entry will appear in the diary listing,
+but will not be marked in the calendar.
+
+This function is provided for use with `diary-nongregorian-listing-hook'.
+
+\(fn)" nil nil)
+
+(autoload 'diary-chinese-anniversary "cal-china" "\
+Like `diary-anniversary' (which see) but accepts Chinese date.
+
+\(fn MONTH DAY &optional YEAR MARK)" nil nil)
+
+(autoload 'diary-chinese-insert-anniversary-entry "cal-china" "\
+Insert an anniversary diary entry for the Chinese date at point.
+Prefix argument ARG makes the entry nonmarking.
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'diary-chinese-insert-entry "cal-china" "\
+Insert a diary entry for the Chinese date at point.
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'diary-chinese-insert-monthly-entry "cal-china" "\
+Insert a monthly diary entry for the Chinese date at point.
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'diary-chinese-insert-yearly-entry "cal-china" "\
+Insert a yearly diary entry for the Chinese date at point.
+
+\(fn &optional ARG)" t nil)
+
 ;;;***
 
-;;;### (autoloads (calendar-ethiopic-goto-date calendar-ethiopic-print-date
-;;;;;;  calendar-ethiopic-date-string calendar-coptic-goto-date calendar-coptic-print-date
-;;;;;;  calendar-coptic-date-string) "cal-coptic" "cal-coptic.el"
-;;;;;;  (19383 49278))
+;;;### (autoloads nil "cal-coptic" "cal-coptic.el" (22676 23208 558522
+;;;;;;  934000))
 ;;; Generated autoloads from cal-coptic.el
 
 (autoload 'calendar-coptic-date-string "cal-coptic" "\
@@ -116,9 +158,8 @@ Echo Ethiopic date unless NOECHO is t.
 
 ;;;***
 
-;;;### (autoloads (calendar-french-goto-date calendar-french-print-date
-;;;;;;  calendar-french-date-string) "cal-french" "cal-french.el"
-;;;;;;  (19383 49278))
+;;;### (autoloads nil "cal-french" "cal-french.el" (22676 23208 558522
+;;;;;;  934000))
 ;;; Generated autoloads from cal-french.el
 
 (autoload 'calendar-french-date-string "cal-french" "\
@@ -141,10 +182,8 @@ Echo French Revolutionary date unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads (diary-hebrew-insert-yearly-entry diary-hebrew-insert-monthly-entry
-;;;;;;  diary-hebrew-insert-entry calendar-hebrew-goto-date calendar-hebrew-print-date
-;;;;;;  calendar-hebrew-date-string) "cal-hebrew" "cal-hebrew.el"
-;;;;;;  (19383 49284))
+;;;### (autoloads nil "cal-hebrew" "cal-hebrew.el" (22676 23208 558522
+;;;;;;  934000))
 ;;; Generated autoloads from cal-hebrew.el
 
 (autoload 'calendar-hebrew-date-string "cal-hebrew" "\
@@ -186,8 +225,8 @@ Prefix argument ARG makes the entry nonmarking.
 
 ;;;***
 
-;;;### (autoloads (cal-html-cursor-year cal-html-cursor-month) "cal-html"
-;;;;;;  "cal-html.el" (19383 49278))
+;;;### (autoloads nil "cal-html" "cal-html.el" (22676 23208 559522
+;;;;;;  930000))
 ;;; Generated autoloads from cal-html.el
 
 (autoload 'cal-html-cursor-month "cal-html" "\
@@ -209,10 +248,8 @@ specified by EVENT.  Note that any existing output files are overwritten.
 
 ;;;***
 
-;;;### (autoloads (diary-islamic-insert-yearly-entry diary-islamic-insert-monthly-entry
-;;;;;;  diary-islamic-insert-entry calendar-islamic-goto-date calendar-islamic-print-date
-;;;;;;  calendar-islamic-date-string) "cal-islam" "cal-islam.el"
-;;;;;;  (19383 49278))
+;;;### (autoloads nil "cal-islam" "cal-islam.el" (22676 23208 559522
+;;;;;;  930000))
 ;;; Generated autoloads from cal-islam.el
 
 (autoload 'calendar-islamic-date-string "cal-islam" "\
@@ -256,9 +293,8 @@ Prefix argument ARG makes the entry nonmarking.
 
 ;;;***
 
-;;;### (autoloads (calendar-iso-goto-week calendar-iso-goto-date
-;;;;;;  calendar-iso-print-date calendar-iso-date-string calendar-iso-from-absolute)
-;;;;;;  "cal-iso" "cal-iso.el" (19383 49278))
+;;;### (autoloads nil "cal-iso" "cal-iso.el" (22676 23208 559522
+;;;;;;  930000))
 ;;; Generated autoloads from cal-iso.el
 
 (autoload 'calendar-iso-from-absolute "cal-iso" "\
@@ -296,11 +332,8 @@ Interactively, goes to the first day of the specified week.
 
 ;;;***
 
-;;;### (autoloads (calendar-astro-goto-day-number calendar-astro-print-day-number
-;;;;;;  calendar-astro-date-string calendar-astro-from-absolute calendar-astro-to-absolute
-;;;;;;  calendar-julian-goto-date calendar-julian-print-date calendar-julian-date-string
-;;;;;;  calendar-julian-from-absolute) "cal-julian" "cal-julian.el"
-;;;;;;  (19383 49278))
+;;;### (autoloads nil "cal-julian" "cal-julian.el" (22676 23208 559522
+;;;;;;  930000))
 ;;; Generated autoloads from cal-julian.el
 
 (autoload 'calendar-julian-from-absolute "cal-julian" "\
@@ -356,11 +389,8 @@ Echo astronomical (Julian) day number unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads (calendar-mayan-goto-long-count-date calendar-mayan-previous-round-date
-;;;;;;  calendar-mayan-next-round-date calendar-mayan-previous-tzolkin-date
-;;;;;;  calendar-mayan-next-tzolkin-date calendar-mayan-previous-haab-date
-;;;;;;  calendar-mayan-next-haab-date calendar-mayan-print-date calendar-mayan-date-string)
-;;;;;;  "cal-mayan" "cal-mayan.el" (19383 49278))
+;;;### (autoloads nil "cal-mayan" "cal-mayan.el" (22676 23208 559522
+;;;;;;  930000))
 ;;; Generated autoloads from cal-mayan.el
 
 (autoload 'calendar-mayan-date-string "cal-mayan" "\
@@ -417,15 +447,8 @@ Echo Mayan date unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads (calendar-goto-day-of-year calendar-goto-date calendar-end-of-year
-;;;;;;  calendar-beginning-of-year calendar-end-of-month calendar-beginning-of-month
-;;;;;;  calendar-end-of-week calendar-beginning-of-week calendar-backward-week
-;;;;;;  calendar-forward-week calendar-backward-day calendar-forward-day
-;;;;;;  calendar-scroll-right-three-months calendar-scroll-left-three-months
-;;;;;;  calendar-scroll-right calendar-scroll-left calendar-backward-year
-;;;;;;  calendar-backward-month calendar-forward-year calendar-forward-month
-;;;;;;  calendar-goto-today calendar-cursor-to-visible-date calendar-cursor-to-nearest-date)
-;;;;;;  "cal-move" "cal-move.el" (19383 49278))
+;;;### (autoloads nil "cal-move" "cal-move.el" (22676 23208 559522
+;;;;;;  930000))
 ;;; Generated autoloads from cal-move.el
 
 (autoload 'calendar-cursor-to-nearest-date "cal-move" "\
@@ -493,6 +516,11 @@ EVENT is an event like `last-nonmenu-event'.
 
 \(fn ARG &optional EVENT)" t nil)
 
+(autoload 'calendar-scroll-toolkit-scroll "cal-move" "\
+Function to scroll the calendar after a toolkit scroll-bar click.
+
+\(fn EVENT)" t nil)
+
 (autoload 'calendar-scroll-right-three-months "cal-move" "\
 Scroll the displayed calendar window right by 3*ARG months.
 If ARG is negative the calendar is scrolled left.  Maintains the relative
@@ -568,9 +596,8 @@ Negative DAY counts backward from end of year.
 
 ;;;***
 
-;;;### (autoloads (calendar-persian-goto-date calendar-persian-print-date
-;;;;;;  calendar-persian-date-string) "cal-persia" "cal-persia.el"
-;;;;;;  (19383 49278))
+;;;### (autoloads nil "cal-persia" "cal-persia.el" (22676 23208 559522
+;;;;;;  930000))
 ;;; Generated autoloads from cal-persia.el
 
 (autoload 'calendar-persian-date-string "cal-persia" "\
@@ -591,12 +618,8 @@ Echo Persian date unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads (cal-tex-cursor-day cal-tex-cursor-filofax-daily
-;;;;;;  cal-tex-cursor-filofax-week cal-tex-cursor-filofax-2week
-;;;;;;  cal-tex-cursor-week-monday cal-tex-cursor-week-iso cal-tex-cursor-week2
-;;;;;;  cal-tex-cursor-week cal-tex-cursor-month cal-tex-cursor-month-landscape
-;;;;;;  cal-tex-cursor-filofax-year cal-tex-cursor-year-landscape
-;;;;;;  cal-tex-cursor-year) "cal-tex" "cal-tex.el" (19383 49284))
+;;;### (autoloads nil "cal-tex" "cal-tex.el" (22676 23208 560522
+;;;;;;  926000))
 ;;; Generated autoloads from cal-tex.el
 
 (autoload 'cal-tex-cursor-year "cal-tex" "\
@@ -641,42 +664,68 @@ indicates a buffer position to use instead of point.
 \(fn &optional N EVENT)" t nil)
 
 (autoload 'cal-tex-cursor-week "cal-tex" "\
-Make a LaTeX calendar buffer for a two-page one-week calendar.
-It applies to the week that point is in.  The optional prefix
-argument N specifies number of weeks (default 1).  The calendar
-shows holidays if `cal-tex-holidays' is non-nil (note that diary
-entries are not shown).  The calendar shows the hours 8-12am, 1-5pm.
+Make a one page LaTeX calendar for one week, showing hours of the day.
+There are two columns; with 8-12am in the first and 1-5pm in the second.
+It shows holidays if `cal-tex-holidays' is non-nil.
+It does not show diary entries.
+
+The optional prefix argument N specifies a number of weeks (default 1).
+
+By default, the calendar is for the week at point; the optional
+argument EVENT specifies a different buffer position.
 
 \(fn &optional N EVENT)" t nil)
 
 (autoload 'cal-tex-cursor-week2 "cal-tex" "\
-Make a LaTeX calendar buffer for a two-page one-week calendar.
-It applies to the week that point is in.  Optional prefix
-argument N specifies number of weeks (default 1).  The calendar
-shows holidays if `cal-tex-holidays' is non-nil (note that diary
-entries are not shown).  The calendar shows the hours 8-12am, 1-5pm.
-Optional EVENT indicates a buffer position to use instead of point.
+Make a two page LaTeX calendar for one week, showing hours of the day.
+There are two columns; with 8-12am in the first and 1-5pm in the second.
+It shows holidays if `cal-tex-holidays' is non-nil.
+It does not show diary entries.
+
+The optional prefix argument N specifies a number of weeks (default 1).
+
+By default, the calendar is for the week at point; the optional
+argument EVENT specifies a different buffer position.
 
 \(fn &optional N EVENT)" t nil)
 
 (autoload 'cal-tex-cursor-week-iso "cal-tex" "\
-Make a LaTeX calendar buffer for a one page ISO-style weekly calendar.
-Optional prefix argument N specifies number of weeks (default 1).
-The calendar shows holiday and diary entries if
-`cal-tex-holidays' and `cal-tex-diary', respectively, are non-nil.
-It does not show hours of the day.  Optional EVENT indicates a buffer
-position to use instead of point.
+Make a one page LaTeX calendar for one week, in the ISO-style.
+It does not show hours of the day.
+It shows holidays if `cal-tex-holidays' is non-nil.
+It shows diary entries if `cal-tex-diary' is non-nil.
+
+The optional prefix argument N specifies a number of weeks (default 1).
+
+By default, the calendar is for the week at point; the optional
+argument EVENT specifies a different buffer position.
 
 \(fn &optional N EVENT)" t nil)
 
 (autoload 'cal-tex-cursor-week-monday "cal-tex" "\
-Make a LaTeX calendar buffer for a two-page one-week calendar.
-It applies to the week that point is in, and starts on Monday.
-Optional prefix argument N specifies number of weeks (default 1).
-The calendar shows holidays if `cal-tex-holidays' is
-non-nil (note that diary entries are not shown).   The calendar shows
-the hours 8-12am, 1-5pm.  Optional EVENT indicates a buffer position
-to use instead of point.
+Make a one page LaTeX calendar for one week, showing hours of the day.
+There are two columns; with M-W in the first and T-S in the second.
+It shows the hours 8-12am and 1-5pm.
+It shows holidays if `cal-tex-holidays' is non-nil.
+It does not show diary entries.
+
+The optional prefix argument N specifies a number of weeks (default 1).
+
+By default, the calendar is for the week at point; the optional
+argument EVENT specifies a different buffer position.
+
+\(fn &optional N EVENT)" t nil)
+
+(autoload 'cal-tex-cursor-week2-summary "cal-tex" "\
+Make a two page LaTeX calendar for one week, with optional diary entries.
+It does not show hours of the day.
+It shows holidays if `cal-tex-holidays' is non-nil.
+It shows diary entries if `cal-tex-diary' is non-nil.
+
+The optional prefix argument N specifies a number of weeks (default 1).
+
+By default, the calendar is for the week at point; the optional
+argument EVENT specifies a different buffer position.
 
 \(fn &optional N EVENT)" t nil)
 
@@ -719,9 +768,7 @@ a buffer position to use instead of point.
 
 ;;;***
 
-;;;### (autoloads (calendar-two-frame-setup calendar-only-one-frame-setup
-;;;;;;  calendar-one-frame-setup calendar-frame-setup) "cal-x" "cal-x.el"
-;;;;;;  (19383 49278))
+;;;### (autoloads nil "cal-x" "cal-x.el" (22676 23208 560522 926000))
 ;;; Generated autoloads from cal-x.el
 
 (autoload 'calendar-frame-setup "cal-x" "\
@@ -738,36 +785,15 @@ If PROMPT is non-nil, prompt for the month and year to use.
 
 \(fn CONFIG &optional PROMPT)" nil nil)
 
-(autoload 'calendar-one-frame-setup "cal-x" "\
-Display calendar and diary in a single dedicated frame.
-See `calendar-frame-setup' for more information.
-
-\(fn &optional PROMPT)" nil nil)
-
-(autoload 'calendar-only-one-frame-setup "cal-x" "\
-Display calendar in a dedicated frame.
-See `calendar-frame-setup' for more information.
-
-\(fn &optional PROMPT)" nil nil)
-
-(autoload 'calendar-two-frame-setup "cal-x" "\
-Display calendar and diary in separate, dedicated frames.
-See `calendar-frame-setup' for more information.
-
-\(fn &optional PROMPT)" nil nil)
-
 ;;;***
 
-;;;### (autoloads (diary-font-lock-keywords diary-insert-cyclic-entry
-;;;;;;  diary-insert-block-entry diary-insert-anniversary-entry diary-insert-yearly-entry
-;;;;;;  diary-insert-monthly-entry diary-insert-weekly-entry diary-insert-entry
-;;;;;;  diary-make-entry diary-mark-entries diary-show-all-entries
-;;;;;;  diary-view-other-diary-entries diary-view-entries diary-set-maybe-redraw
-;;;;;;  diary-live-p) "diary-lib" "diary-lib.el" (19383 49276))
+;;;### (autoloads nil "diary-lib" "diary-lib.el" (22676 23208 562522
+;;;;;;  918000))
 ;;; Generated autoloads from diary-lib.el
 
 (autoload 'diary-live-p "diary-lib" "\
 Return non-nil if the diary is being displayed.
+The actual return value is a diary buffer.
 
 \(fn)" nil nil)
 
@@ -794,8 +820,6 @@ DFILE specifies the file to use as the diary file.
 
 \(fn ARG DFILE)" t nil)
 
-(define-obsolete-function-alias 'view-other-diary-entries 'diary-view-other-diary-entries "23.1")
-
 (autoload 'diary-show-all-entries "diary-lib" "\
 Show all of the diary entries in the diary file.
 This function gets rid of the selective display of the diary file so that
@@ -806,18 +830,21 @@ is created.
 
 (autoload 'diary-mark-entries "diary-lib" "\
 Mark days in the calendar window that have diary entries.
-Each entry in the diary file visible in the calendar window is
-marked.  After the entries are marked, the hooks
-`diary-nongregorian-marking-hook' and `diary-mark-entries-hook'
-are run.  If the optional argument REDRAW is non-nil (which is
-the case interactively, for example) then any existing diary
-marks are first removed.  This is intended to deal with deleted
-diary entries.
+Marks each entry in the diary that is visible in the calendar window.
+
+After marking the entries, runs `diary-nongregorian-marking-hook'
+for the main diary file, and each included file.  For example,
+this is the appropriate hook to process Islamic entries in all
+diary files.  Next `diary-mark-entries-hook' runs, for the main diary
+file only.  If not using include files, there is no difference between
+these two hooks.
+
+If the optional argument REDRAW is non-nil (which is the case
+interactively, for example) then this first removes any existing diary
+marks.  This is intended to deal with deleted diary entries.
 
 \(fn &optional REDRAW)" t nil)
 
-(define-obsolete-function-alias 'mark-diary-entries 'diary-mark-entries "23.1")
-
 (autoload 'diary-make-entry "diary-lib" "\
 Insert a diary entry STRING which may be NONMARKING in FILE.
 If omitted, NONMARKING defaults to nil and FILE defaults to
@@ -825,64 +852,48 @@ If omitted, NONMARKING defaults to nil and FILE defaults to
 
 \(fn STRING &optional NONMARKING FILE)" nil nil)
 
-(define-obsolete-function-alias 'make-diary-entry 'diary-make-entry "23.1")
-
 (autoload 'diary-insert-entry "diary-lib" "\
 Insert a diary entry for the date indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG &optional EVENT)" t nil)
 
-(define-obsolete-function-alias 'insert-diary-entry 'diary-insert-entry "23.1")
-
 (autoload 'diary-insert-weekly-entry "diary-lib" "\
 Insert a weekly diary entry for the day of the week indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-weekly-diary-entry 'diary-insert-weekly-entry "23.1")
-
 (autoload 'diary-insert-monthly-entry "diary-lib" "\
 Insert a monthly diary entry for the day of the month indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-monthly-diary-entry 'diary-insert-monthly-entry "23.1")
-
 (autoload 'diary-insert-yearly-entry "diary-lib" "\
 Insert an annual diary entry for the day of the year indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-yearly-diary-entry 'diary-insert-yearly-entry "23.1")
-
 (autoload 'diary-insert-anniversary-entry "diary-lib" "\
 Insert an anniversary diary entry for the date given by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-anniversary-diary-entry 'diary-insert-anniversary-entry "23.1")
-
 (autoload 'diary-insert-block-entry "diary-lib" "\
 Insert a block diary entry for the days between the point and marked date.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-block-diary-entry 'diary-insert-block-entry "23.1")
-
 (autoload 'diary-insert-cyclic-entry "diary-lib" "\
 Insert a cyclic diary entry starting at the date given by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-cyclic-diary-entry 'diary-insert-cyclic-entry "23.1")
-
 (autoload 'diary-font-lock-keywords "diary-lib" "\
 Return a value for the variable `diary-font-lock-keywords'.
 
@@ -890,8 +901,8 @@ Return a value for the variable `diary-font-lock-keywords'.
 
 ;;;***
 
-;;;### (autoloads (calendar-mark-holidays calendar-cursor-holidays
-;;;;;;  calendar-list-holidays) "holidays" "holidays.el" (19383 49279))
+;;;### (autoloads nil "holidays" "holidays.el" (22676 23208 562522
+;;;;;;  918000))
 ;;; Generated autoloads from holidays.el
 
 (autoload 'calendar-list-holidays "holidays" "\
@@ -919,8 +930,7 @@ use instead of point.
 
 ;;;***
 
-;;;### (autoloads (calendar-lunar-phases) "lunar" "lunar.el" (19383
-;;;;;;  49279))
+;;;### (autoloads nil "lunar" "lunar.el" (22676 23208 563522 913000))
 ;;; Generated autoloads from lunar.el
 
 (autoload 'calendar-lunar-phases "lunar" "\
@@ -930,12 +940,9 @@ use instead of point.
 
 \(fn &optional EVENT)" t nil)
 
-(define-obsolete-function-alias 'calendar-phases-of-moon 'calendar-lunar-phases "23.1")
-
 ;;;***
 
-;;;### (autoloads (calendar-sunrise-sunset-month calendar-sunrise-sunset)
-;;;;;;  "solar" "solar.el" (19383 49282))
+;;;### (autoloads nil "solar" "solar.el" (22676 23208 563522 913000))
 ;;; Generated autoloads from solar.el
 
 (autoload 'calendar-sunrise-sunset "solar" "\
@@ -953,7 +960,7 @@ Local time of sunrise and sunset for month under cursor or at EVENT.
 
 ;;;### (autoloads nil nil ("appt.el" "cal-dst.el" "cal-menu.el" "calendar.el"
 ;;;;;;  "icalendar.el" "parse-time.el" "time-date.el" "timeclock.el"
-;;;;;;  "todo-mode.el") (19428 55158 980253))
+;;;;;;  "todo-mode.el") (22676 23208 565522 906000))
 
 ;;;***
 
