@@ -1,10 +1,10 @@
 ;;; eudc.el --- Emacs Unified Directory Client
 
-;; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;; Copyright (C) 1998-2017 Free Software Foundation, Inc.
 
 ;; Author: Oscar Figueiredo <oscar@cpe.fr>
-;; Maintainer: Pavel Jank <Pavel@Janik.cz>
+;;         Pavel Janík <Pavel@Janik.cz>
+;; Maintainer: Thomas Fitzsimmons <fitzsim@fitzsim.org>
 ;; Keywords: comm
 
 ;; This file is part of GNU Emacs.
@@ -47,11 +47,11 @@
 
 (require 'wid-edit)
 
+(eval-when-compile (require 'cl-lib))
+
 (eval-and-compile
   (if (not (fboundp 'make-overlay))
-      (require 'overlay))
-  (if (not (fboundp 'unless))
-      (require 'cl)))
+      (require 'overlay)))
 
 (unless (fboundp 'custom-menu-create)
   (autoload 'custom-menu-create "cus-edit"))
@@ -79,10 +79,6 @@
 
 (defvar mode-popup-menu)
 
-;; List of known servers
-;; Alist of (SERVER . PROTOCOL)
-(defvar eudc-server-hotlist nil)
-
 ;; List of variables that have server- or protocol-local bindings
 (defvar eudc-local-vars nil)
 
@@ -111,17 +107,17 @@
 ;; attribute name
 (defvar eudc-protocol-has-default-query-attributes nil)
 
-(defun eudc-cadr (obj)
-  (car (cdr obj)))
+(defvar bbdb-version)
 
-(defun eudc-cdar (obj)
-  (cdr (car obj)))
-
-(defun eudc-caar (obj)
-  (car (car obj)))
-
-(defun eudc-cdaar (obj)
-  (cdr (car (car obj))))
+(defun eudc--using-bbdb-3-or-newer-p ()
+  "Return non-nil if BBDB version is 3 or greater."
+  (or
+   ;; MELPA versions of BBDB may have a bad package version, but
+   ;; they're all version 3 or later.
+   (equal bbdb-version "@PACKAGE_VERSION@")
+   ;; Development versions of BBDB can have the format "X.YZ devo".
+   ;; Split the string just in case.
+   (version<= "3" (car (split-string bbdb-version)))))
 
 (defun eudc-plist-member (plist prop)
   "Return t if PROP has a value specified in PLIST."
@@ -134,7 +130,7 @@
       (setq plist (cdr (cdr plist))))
     nil))
 
-;; Emacs' plist-get lacks third parameter
+;; Emacs's plist-get lacks third parameter
 (defun eudc-plist-get (plist prop &optional default)
   "Extract a value from a property list.
 PLIST is a property list, which is a list of the form
@@ -352,12 +348,12 @@ accordingly. Otherwise it is set to its EUDC default binding"
 The translation is done according to
 `eudc-protocol-attributes-translation-alist'."
   (if eudc-protocol-attributes-translation-alist
-      (mapcar '(lambda (attribute)
-		 (let ((trans (assq (car attribute)
-				    (symbol-value eudc-protocol-attributes-translation-alist))))
-		   (if trans
-		       (cons (cdr trans) (cdr attribute))
-		     attribute)))
+      (mapcar (lambda (attribute)
+                (let ((trans (assq (car attribute)
+                                   (symbol-value eudc-protocol-attributes-translation-alist))))
+                  (if trans
+                      (cons (cdr trans) (cdr attribute))
+                    attribute)))
 	      query)
     query))
 
@@ -367,7 +363,7 @@ The translation is done according to
 `eudc-protocol-attributes-translation-alist'."
   (if eudc-protocol-attributes-translation-alist
       (let (trans)
-	(mapcar '(lambda (attribute)
+	(mapcar (lambda (attribute)
 		   (setq trans (assq attribute
 				     (symbol-value eudc-protocol-attributes-translation-alist)))
 		   (if trans
@@ -521,12 +517,12 @@ otherwise they are formatted according to `eudc-user-attribute-names-alist'."
 	   precords))
 	(insert "\n")
 	(widget-create 'push-button
-		       :notify (lambda (&rest ignore)
+		       :notify (lambda (&rest _ignore)
 				 (eudc-query-form))
 		       "New query")
 	(widget-insert " ")
 	(widget-create 'push-button
-		       :notify (lambda (&rest ignore)
+		       :notify (lambda (&rest _ignore)
 				 (kill-this-buffer))
 		       "Quit")
 	(eudc-mode)
@@ -561,10 +557,10 @@ otherwise they are formatted according to `eudc-user-attribute-names-alist'."
 
     ;; Search for multiple records
     (while (and rec
-		(not (listp (eudc-cdar rec))))
+		(not (listp (cdar rec))))
       (setq rec (cdr rec)))
 
-    (if (null (eudc-cdar rec))
+    (if (null (cdar rec))
 	(list record)			; No duplicate attrs in this record
       (mapc (function
 	     (lambda (field)
@@ -596,7 +592,7 @@ otherwise they are formatted according to `eudc-user-attribute-names-alist'."
 	     ((eq 'first method)
 	      (setq result
 		    (eudc-add-field-to-records (cons (car field)
-						     (eudc-cadr field))
+						     (cadr field))
 					       result)))
 	     ((eq 'concat method)
 	      (setq result
@@ -655,7 +651,7 @@ Each copy is added a new field containing one of the values of FIELD."
     result))
 
 
-(defun eudc-mode ()
+(define-derived-mode eudc-mode special-mode "EUDC"
   "Major mode used in buffers displaying the results of directory queries.
 There is no sense in calling this command from a buffer other than
 one containing the results of a directory query.
@@ -666,15 +662,9 @@ These are the special commands of EUDC mode:
     n -- Move to next record.
     p -- Move to previous record.
     b -- Insert record at point into the BBDB database."
-  (interactive)
-  (kill-all-local-variables)
-  (setq major-mode 'eudc-mode)
-  (setq mode-name "EUDC")
-  (use-local-map eudc-mode-map)
   (if (not (featurep 'xemacs))
       (easy-menu-define eudc-emacs-menu eudc-mode-map "" (eudc-menu))
-    (setq mode-popup-menu (eudc-menu)))
-  (run-mode-hooks 'eudc-mode-hook))
+    (setq mode-popup-menu (eudc-menu))))
 
 ;;}}}
 
@@ -693,11 +683,12 @@ server for future sessions."
   (interactive (list
 		(read-from-minibuffer "Directory Server: ")
 		(intern (completing-read "Protocol: "
-					 (mapcar '(lambda (elt)
+					 (mapcar (lambda (elt)
 						    (cons (symbol-name elt)
 							  elt))
 						 eudc-known-protocols)))))
-  (unless (or (member protocol
+  (unless (or (null protocol)
+	      (member protocol
 		      eudc-supported-protocols)
 	      (load (concat "eudcb-" (symbol-name protocol)) t))
     (error "Unsupported protocol: %s" protocol))
@@ -706,7 +697,7 @@ server for future sessions."
   (setq eudc-server server)
   (eudc-update-local-variables)
   (run-hooks 'eudc-switch-to-server-hook)
-  (if (interactive-p)
+  (if (called-interactively-p 'interactive)
       (message "Current directory server is now %s (%s)" eudc-server eudc-protocol))
   (if (null no-save)
       (eudc-save-options)))
@@ -721,7 +712,7 @@ If ERROR is non-nil, report an error if there is none."
   (let ((result (eudc-query (list (cons 'name name)) '(email)))
 	email)
     (if (null (cdr result))
-	(setq email (eudc-cdaar result))
+	(setq email (cl-cdaar result))
       (error "Multiple match--use the query form"))
     (if error
 	(if email
@@ -739,7 +730,7 @@ If ERROR is non-nil, report an error if there is none."
   (let ((result (eudc-query (list (cons 'name name)) '(phone)))
 	phone)
     (if (null (cdr result))
-	(setq phone (eudc-cdaar result))
+	(setq phone (cl-cdaar result))
       (error "Multiple match--use the query form"))
     (if error
 	(if phone
@@ -753,9 +744,10 @@ When called interactively the list is formatted in a dedicated buffer
 otherwise a list of symbols is returned."
   (interactive)
   (if eudc-list-attributes-function
-      (let ((entries (funcall eudc-list-attributes-function (interactive-p))))
+      (let ((entries (funcall eudc-list-attributes-function
+			      (called-interactively-p 'interactive))))
 	(if entries
-	    (if (interactive-p)
+	    (if (called-interactively-p 'interactive)
 		(eudc-display-records entries t)
 	      entries)))
     (error "The %s protocol has no support for listing attributes" eudc-protocol)))
@@ -774,10 +766,9 @@ otherwise a list of symbols is returned."
 		  format (cdr format)))
 	  ;; If the same attribute appears more than once, merge
 	  ;; the corresponding values
-	  (setq query-alist (nreverse query-alist))
 	  (while query-alist
-	    (setq key (eudc-caar query-alist)
-		  val (eudc-cdar query-alist)
+	    (setq key (caar query-alist)
+		  val (cdar query-alist)
 		  cell (assq key query))
 	    (if cell
 		(setcdr cell (concat (cdr cell) " " val))
@@ -796,7 +787,7 @@ If none try N - 1 and so forth."
 		(> n 0))
       (setq formats
 	    (delq nil
-		  (mapcar '(lambda (format)
+		  (mapcar (lambda (format)
 			     (if (= n
 				    (length format))
 				 format
@@ -820,22 +811,29 @@ If REPLACE is non-nil, then this expansion replaces the name in the buffer.
 Multiple servers can be tried with the same query until one finds a match,
 see `eudc-inline-expansion-servers'"
   (interactive)
-  (if (memq eudc-inline-expansion-servers
-	    '(current-server server-then-hotlist))
-      (or eudc-server
-	  (call-interactively 'eudc-set-server))
+  (cond
+   ((eq eudc-inline-expansion-servers 'current-server)
+    (or eudc-server
+	(call-interactively 'eudc-set-server)))
+   ((eq eudc-inline-expansion-servers 'server-then-hotlist)
+    (or eudc-server
+	;; Allow server to be nil if hotlist is set.
+	eudc-server-hotlist
+	(call-interactively 'eudc-set-server)))
+   ((eq eudc-inline-expansion-servers 'hotlist)
     (or eudc-server-hotlist
 	(error "No server in the hotlist")))
+   (t
+    (error "Wrong value for `eudc-inline-expansion-servers': %S"
+	   eudc-inline-expansion-servers)))
   (let* ((end (point))
 	 (beg (save-excursion
 		(if (re-search-backward "\\([:,]\\|^\\)[ \t]*"
-					(save-excursion
-					  (beginning-of-line)
-					  (point))
-					'move)
+					(point-at-bol) 'move)
 		    (goto-char (match-end 0)))
 		(point)))
-	 (query-words (split-string (buffer-substring beg end) "[ \t]+"))
+	 (query-words (split-string (buffer-substring-no-properties beg end)
+				    "[ \t]+"))
 	 query-formats
 	 response
 	 response-string
@@ -851,24 +849,23 @@ see `eudc-inline-expansion-servers'"
 	   ((eq eudc-inline-expansion-servers 'hotlist)
 	    eudc-server-hotlist)
 	   ((eq eudc-inline-expansion-servers 'server-then-hotlist)
-	    (cons (cons eudc-server eudc-protocol)
-		  (delete (cons eudc-server eudc-protocol) servers)))
+	    (if eudc-server
+		(cons (cons eudc-server eudc-protocol)
+		      (delete (cons eudc-server eudc-protocol) servers))
+	      eudc-server-hotlist))
 	   ((eq eudc-inline-expansion-servers 'current-server)
-	    (list (cons eudc-server eudc-protocol)))
-	   (t
-	    (error "Wrong value for `eudc-inline-expansion-servers': %S"
-		   eudc-inline-expansion-servers))))
+	    (list (cons eudc-server eudc-protocol)))))
     (if (and eudc-max-servers-to-query
 	     (> (length servers) eudc-max-servers-to-query))
 	(setcdr (nthcdr (1- eudc-max-servers-to-query) servers) nil))
 
-    (condition-case signal
+    (unwind-protect
 	(progn
 	  (setq response
 		(catch 'found
 		  ;; Loop on the servers
 		  (while servers
-		    (eudc-set-server (eudc-caar servers) (eudc-cdar servers) t)
+		    (eudc-set-server (caar servers) (cdar servers) t)
 
 		    ;; Determine which formats apply in the query-format list
 		    (setq query-formats
@@ -898,14 +895,15 @@ see `eudc-inline-expansion-servers'"
 
 	    ;; Process response through eudc-inline-expansion-format
 	    (while response
-	      (setq response-string (apply 'format
-					   (car eudc-inline-expansion-format)
-					   (mapcar (function
-						    (lambda (field)
-						      (or (cdr (assq field (car response)))
-							  "")))
-						   (eudc-translate-attribute-list
-						    (cdr eudc-inline-expansion-format)))))
+	      (setq response-string
+                    (apply 'format
+                           (car eudc-inline-expansion-format)
+                           (mapcar (function
+                                    (lambda (field)
+                                      (or (cdr (assq field (car response)))
+                                          "")))
+                                   (eudc-translate-attribute-list
+                                    (cdr eudc-inline-expansion-format)))))
 	      (if (> (length response-string) 0)
 		  (setq response-strings
 			(cons response-string response-strings)))
@@ -927,15 +925,10 @@ see `eudc-inline-expansion-servers'"
 	      (delete-region beg end)
 	      (insert (mapconcat 'identity response-strings ", ")))
 	     ((eq eudc-multiple-match-handling-method 'abort)
-	      (error "There is more than one match for the query"))))
-	  (or (and (equal eudc-server eudc-former-server)
-		   (equal eudc-protocol eudc-former-protocol))
-	      (eudc-set-server eudc-former-server eudc-former-protocol t)))
-      (t
-       (or (and (equal eudc-server eudc-former-server)
-		(equal eudc-protocol eudc-former-protocol))
-	   (eudc-set-server eudc-former-server eudc-former-protocol t))
-       (signal (car signal) (cdr signal))))))
+	      (error "There is more than one match for the query")))))
+      (or (and (equal eudc-server eudc-former-server)
+	       (equal eudc-protocol eudc-former-protocol))
+	  (eudc-set-server eudc-former-server eudc-former-protocol t)))))
 
 ;;;###autoload
 (defun eudc-query-form (&optional get-fields-from-server)
@@ -1000,17 +993,17 @@ queries the server for the existing fields and displays a corresponding form."
 	  fields)
     (widget-insert "\n\n")
     (widget-create 'push-button
-		   :notify (lambda (&rest ignore)
+		   :notify (lambda (&rest _ignore)
 			     (eudc-process-form))
 		   "Query Server")
     (widget-insert " ")
     (widget-create 'push-button
-		   :notify (lambda (&rest ignore)
+		   :notify (lambda (&rest _ignore)
 			     (eudc-query-form))
 		   "Reset Form")
     (widget-insert " ")
     (widget-create 'push-button
-		   :notify (lambda (&rest ignore)
+		   :notify (lambda (&rest _ignore)
 			     (kill-this-buffer))
 		   "Quit")
     (goto-char pt)
@@ -1035,8 +1028,7 @@ queries the server for the existing fields and displays a corresponding form."
 (defun eudc-save-options ()
   "Save options to `eudc-options-file'."
   (interactive)
-  (save-excursion
-    (set-buffer (find-file-noselect eudc-options-file t))
+  (with-current-buffer (find-file-noselect eudc-options-file t)
     (goto-char (point-min))
     ;; delete the previous setq
     (let ((standard-output (current-buffer))
@@ -1057,14 +1049,14 @@ queries the server for the existing fields and displays a corresponding form."
 				 (point))
 		  (setq set-server-p t))
 		 ((and (eq (car sexp)  'setq)
-		       (eq (eudc-cadr sexp) 'eudc-server-hotlist))
+		       (eq (cadr sexp) 'eudc-server-hotlist))
 		  (delete-region (save-excursion
 				   (backward-sexp)
 				   (point))
 				 (point))
 		  (setq set-hotlist-p t))
 		 ((and (eq (car sexp)  'provide)
-		       (equal (eudc-cadr sexp) '(quote eudc-options-file)))
+		       (equal (cadr sexp) '(quote eudc-options-file)))
 		  (setq provide-p t)))
 	      (if (and provide-p
 		       set-hotlist-p
@@ -1090,7 +1082,7 @@ queries the server for the existing fields and displays a corresponding form."
 (defun eudc-move-to-next-record ()
   "Move to next record, in a buffer displaying directory query results."
   (interactive)
-  (if (not (eq major-mode 'eudc-mode))
+  (if (not (derived-mode-p 'eudc-mode))
       (error "Not in a EUDC buffer")
     (let ((pt (next-overlay-change (point))))
       (if (< pt (point-max))
@@ -1100,7 +1092,7 @@ queries the server for the existing fields and displays a corresponding form."
 (defun eudc-move-to-previous-record ()
   "Move to previous record, in a buffer displaying directory query results."
   (interactive)
-  (if (not (eq major-mode 'eudc-mode))
+  (if (not (derived-mode-p 'eudc-mode))
       (error "Not in a EUDC buffer")
     (let ((pt (previous-overlay-change (point))))
       (if (> pt (point-min))
@@ -1128,7 +1120,7 @@ queries the server for the existing fields and displays a corresponding form."
 	  (overlay-get (car (overlays-at (point))) 'eudc-record))
      :help "Insert record at point into the BBDB database"]
     ["Insert All Records into BBDB" eudc-batch-export-records-to-bbdb
-     (and (eq major-mode 'eudc-mode)
+     (and (derived-mode-p 'eudc-mode)
 	  (or (featurep 'bbdb)
 	      (prog1 (locate-library "bbdb") (message ""))))
      :help "Insert all the records returned by a directory query into BBDB"]
@@ -1217,7 +1209,7 @@ queries the server for the existing fields and displays a corresponding form."
 ;;; Load the options file
 (if (and (not noninteractive)
 	 (and (locate-library eudc-options-file)
-	      (progn (message "") t))   ; Remove modeline message
+	      (progn (message "") t))   ; Remove mode line message
 	 (not (featurep 'eudc-options-file)))
     (load eudc-options-file))
 
@@ -1241,25 +1233,25 @@ This does nothing except loading eudc by autoload side-effect."
   (defvar eudc-tools-menu
     (let ((map (make-sparse-keymap "Directory Search")))
       (define-key map [phone]
-	'(menu-item "Get Phone" eudc-get-phone
-		    :help "Get the phone field of name from the directory server"))
+	`(menu-item ,(purecopy "Get Phone") eudc-get-phone
+		    :help ,(purecopy "Get the phone field of name from the directory server")))
       (define-key map [email]
-	'(menu-item "Get Email" eudc-get-email
-		    :help "Get the email field of NAME from the directory server"))
-      (define-key map [separator-eudc-email] '("--"))
+	`(menu-item ,(purecopy "Get Email") eudc-get-email
+		    :help ,(purecopy "Get the email field of NAME from the directory server")))
+      (define-key map [separator-eudc-email] menu-bar-separator)
       (define-key map [expand-inline]
-	'(menu-item "Expand Inline Query" eudc-expand-inline
-		    :help "Query the directory server, and expand the query string before point"))
+	`(menu-item ,(purecopy "Expand Inline Query") eudc-expand-inline
+		    :help ,(purecopy "Query the directory server, and expand the query string before point")))
       (define-key map [query]
-	'(menu-item "Query with Form" eudc-query-form
-		    :help "Display a form to query the directory server"))
-      (define-key map [separator-eudc-query] '("--"))
+	`(menu-item ,(purecopy "Query with Form") eudc-query-form
+		    :help ,(purecopy "Display a form to query the directory server")))
+      (define-key map [separator-eudc-query] menu-bar-separator)
       (define-key map [new]
-	'(menu-item "New Server" eudc-set-server
-		    :help "Set the directory server to SERVER using PROTOCOL"))
+	`(menu-item ,(purecopy "New Server") eudc-set-server
+		    :help ,(purecopy "Set the directory server to SERVER using PROTOCOL")))
       (define-key map [load]
-	'(menu-item "Load Hotlist of Servers" eudc-load-eudc
-		    :help "Load the Emacs Unified Directory Client"))
+	`(menu-item ,(purecopy "Load Hotlist of Servers") eudc-load-eudc
+		    :help ,(purecopy "Load the Emacs Unified Directory Client")))
       map))
   (fset 'eudc-tools-menu (symbol-value 'eudc-tools-menu)))
  (t
@@ -1295,5 +1287,4 @@ This does nothing except loading eudc by autoload side-effect."
 
 (provide 'eudc)
 
-;; arch-tag: e18872b6-db83-400b-869d-be54e9a4160c
 ;;; eudc.el ends here
